/**
 * Copyright 2005 Jason LaDere
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Originally based on Zilmar's Basic CFB Plugin.
 *
 */

package cfb;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.nio.ByteBuffer;
import java.nio.IntBuffer;
import javax.swing.JFrame;
import plugin.GfxPlugin;

/**
 *
 * @author Jason
 */
public class CFBPlugin implements GfxPlugin {
    
    public static final boolean DEBUG = false;
    
    private Graphics2D ddPrimary;
    private BufferedImage ddCFB;
    private boolean fullScreen;
    private int[] convet16to32 = new int[65536];
    private int viStatus;
    private int viWidth;
    private String name;
    
    private JFrame hWnd;
    private int[] viReg;
    private ByteBuffer rdram;
    private boolean memoryBswaped;
    
    public CFBPlugin() {
        name = this.getClass().getName();
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") created.");
    }

    public void captureScreen(String directory) {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") captureScreen.");
    }

    public void changeWindow() {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") changeWindow.");
    }

    public void closePlugin() {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") closePlugin.");
    }

    public void pluginAbout(JFrame hParent) {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") pluginAbout.");
    }

    public void pluginConfig(JFrame hParent) {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") pluginConfig.");
    }

    public void pluginTest(JFrame hParent) {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") pluginTest.");
    }

    public void drawScreen() {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") drawScreen.");
        if (ddCFB == null)
            return;
        if (ddPrimary == null)
            return;
        ddPrimary.drawImage(ddCFB, 0, 0, null);
    }

    public void getPluginInfo(PluginInfo pluginInfo) {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") getPluginInfo.");
        pluginInfo.version = 0x0102;
        pluginInfo.type = PLUGIN_TYPE_GFX;
        pluginInfo.name = "Basic CFB plugin by zilmar";
        pluginInfo.normalMemory = true;
        pluginInfo.memoryBswaped = true;
    }

    public boolean initiateGfx(GfxInfo gfxInfo) {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") initiateGFX.");
        this.hWnd = gfxInfo.hWnd;
        this.viReg = gfxInfo.viRegisters;
        this.rdram = gfxInfo.rdram;
        this.memoryBswaped = gfxInfo.memoryBswaped;
        
        int red, green, blue, alpha;
        int color16;
        int color32;
        
        fullScreen = false;
        ddPrimary = (Graphics2D)hWnd.getComponent(0).getGraphics();
        
        for (int count = 0; count < 65536; count ++) {
            color16 = count;
            red   = (color16 & 0xF800) >> 11;
            green = (color16 & 0x07C0) >> 6;
            blue  = (color16 & 0x003E) >> 1;
            alpha = (color16 & 0x0001);
            color32 = (alpha<<27) | (red<<19) | (green<<11) | (blue<<3);
            convet16to32[count] = color32;
        }
        
        ddCFB  = null;
        return true;
    }

    public void moveScreen(int xpos, int ypos) {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") moveScreen.");
    }
    
    public void processDList() {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") processDList.");
    }

    public void processRDPList() {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") processRDPList.");
    }

    public void romClosed() {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") romClosed.");
    }

    public void romOpen() {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") romOpen.");
    }

    public void showCFB() {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") showCFB.");
    }

    public void updateScreen() {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") updateScreen.");
        BufferedImage surfBuf;
        IntBuffer buffer;
        if (ddCFB == null)
            return;
        
        rdram.position(viReg[VI_ORIGIN_REG] & 0x00FFFFFF);
        buffer = rdram.asIntBuffer();
        
        int w = viReg[VI_WIDTH_REG];
        int h = ((viReg[VI_WIDTH_REG] >> 2) * 3);
        surfBuf = ddCFB;
        
        for (int count = 0; count < h; count ++) {
            for (int x = 0; x < w; x += 2) {
                if (memoryBswaped) {
                    int c = buffer.get();
                    surfBuf.setRGB(x+1, count, convet16to32[(c>>16)&0xFFFF]);
                    surfBuf.setRGB(x, count, convet16to32[c&0xFFFF]);
                } else {
                    int c = buffer.get();
                    surfBuf.setRGB(x, count, convet16to32[(c>>16)&0xFFFF]);
                    surfBuf.setRGB(x+1, count, convet16to32[c&0xFFFF]);
                }
            }
        }
        
        drawScreen();
        
        return;
    }

    public void viStatusChanged() {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") viStatusChanged.");
        if (viReg[VI_STATUS_REG] != viStatus) {
            setupCFB();
            viStatus = viReg[VI_STATUS_REG];
        }
    }

    public void viWidthChanged() {
        if (DEBUG) System.out.println("GFX Plugin (" + name + ") viWidthChanged.");
        if (viReg[VI_WIDTH_REG] != viWidth) {
            setupCFB();
            viWidth = viReg[VI_WIDTH_REG];
            changeWinSize(hWnd, viWidth);
        }
    }
    
    public int updateCurrentHalfLine(int timer) {
        return 0;
    }
    
    // Private Methods /////////////////////////////////////////////////////////
    
    private void setupCFB() {
        if (viReg[VI_STATUS_REG] == 0)
            return;
        if (viReg[VI_WIDTH_REG] == 0)
            return;
        
        int w = viReg[VI_WIDTH_REG];  
        int h = ((viReg[VI_WIDTH_REG] >> 2) * 3);
        ddCFB = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
    }
    
    private void changeWinSize(JFrame hWnd, int width) {
        if (fullScreen)
            return;
        hWnd.getComponent(0).setPreferredSize(new Dimension(width, (width >> 2) * 3));
        hWnd.pack();
    }
    
}
